import { guid } from '@/uilts/help'
//  iframe 通信中心
class MessageCenter {
    constructor() {
        //  缓存事件队列
        this.messageCallbackMap = new Map();
        this.messageFailbackMap = new Map();
        this.timer = null
        this.init()
    }

    send({ info = {}, data = {}, callback, overTime, failback }) {
        info.messageId = `${info.actionType}-${guid()}` // 唯一的ID，用于标记回调函数
        window.parent.postMessage({
            info,
            data
        }, '*');
        if (info.messageId && callback) {
            // 带回调callback 的message， 要求携带messageId，callback，failback等
            this.listen(info.messageId, callback)
            if (failback) {
                this.addFailback(info.messageId, overTime, failback)
            }
        }
    }

    listen(messageId, callback) {
        let activeQuene = this.messageCallbackMap.get(messageId);
        if (activeQuene?.length > 0) {
            activeQuene.push(callback)
        } else {
            this.messageCallbackMap.set(messageId, [callback])
        }
    }

    addFailback(messageId, overTime = 2000, failback) {
        let failbackQuene = this.messageFailbackMap.get(messageId);
        if (failbackQuene && failbackQuene.failCallbackList) {
            failbackQuene.failCallbackList.push(
                failback
            )
        } else {
            this.messageFailbackMap.set(messageId, {
                time: new Date().getTime(),
                overTime,
                failCallbackList: [failback]
            })
        }
        this.checkTimer()
    }



    init() {
        window.addEventListener('message', (e) => {
            const { info, data } = e.data;
            this.messageFailbackMap.delete(info.messageId);
            const quene = this.messageCallbackMap.get(info.messageId) || [];
            let index = 0;
            while (index < quene.length) {
                const callback = quene[index];
                callback(data)
                index++
            }
        })

        this.checkTimer()
    }
    checkTimer() {
        if (this.timer) {
            return
        }
        this.timer = setInterval(() => {
            if (this.messageFailbackMap.values().length == 0) {
                clearInterval(this.timer)
                this.timer = null
            }

            // 轮询查看有无超期的message信息
            const now = new Date().getTime();
            for (let item of this.messageFailbackMap.values()) {
                if (now - item.time > item.overTime) {
                    let index = 0;
                    while (index < item.failCallbackList.length) {
                        const callback = item.failCallbackList[index];
                        callback({
                            error: 0,
                            msg: "message 超时错误"
                        })
                        index++
                    }
                }
            }
        }, 1000)
    }
}

// messageCenter在每个iframe内实例化一次
const messageCenter = new MessageCenter();

export default messageCenter;